#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string>       
#include <iostream>     
#include <sstream>
#include <assert.h>  

#include "Array1D.h"
#include "Array2D.h"
#include "datafiles.h"
#include "uqtkmcmc.h"
//#include "arrayio.h"

struct llpars {

  int     dayst  ;
  int     dayen  ;
  int     nday   ;
  int     ncycles;
  int     npar   ;
  double *obsdata  ;
  double *obssigma ;
  double *drivers  ;
  double *model    ;

} ;

/* LINPACK */
extern "C" void dgbfa_(double *abd,int *lda,int *n,int *ml,int *mu,int *ipvt,int *info);
extern "C" void dgbsl_(double *abd,int *lda,int *n,int *ml,int *mu,int *ipvt,double *rhs,int *job);
extern "C" void dgbdi_(double *abd,int *lda,int *n,int *ml,int *mu,int *ipvt,double *detSigma);
extern "C" void dgtsl_(int *n, double *c,double *d,double *e,double *rhs,int *info);

/* set matrix in LINPACK format */
extern "C" void setabd_(double *abd,double *diag,double *factrs,int *lda,int *ml,int *mu,int *n);

/* dalec model */
extern "C" void dalec_deciduous_(int *nday,int *ncycles, double *params,double *driver,double *model) ;

/* input driver and observation data */
void readDriverData(char *dfile, int nday, double *dridata) ;
void readObs(char *ofile, int nday, double *obsdata, double *obssigma) ;

/* posterior/prior functions */
double logDalecPost (Array1D<double> &chain, void *params) ;
double logDalecPrior(Array1D<double> &chain, void *params) ;

/* predefined DALEC parameters */
#define NDAY    5844
#define DAYST   366   
#define DAYEN   3365
#define DRIVERFILE "./US-Ha1_drivers.csv"
#define OBSFILE    "./US-Ha1_NEE_daily.txt"

/* Standard values for MCMC setup */
#define FIXST   1000
#define ADPST   100
#define ENDAD   5000000
#define MCMCST  5000000
#define GAMMFAC 0.2
#define SIGFAC  100.0
#define LCORR   0.5

#define SAVE2FILE   10000  // save to file every "SAVE2FILE"   MCMC steps
#define SAVE2SCREEN 1000   // save to file every "SAVE2SCREEN" MCMC steps

#define DPI 6.28318530717959

int main() {

  llpars params ;
  double *drivers, *model, *obsdata, *obssigma ;
  int npar    = NPAR    ;
  int nday    = NDAY    ;
  int ncycles = NCYCLES ;
  int dayst   = DAYST   ;
  int dayen   = DAYEN   ;

  double gamma ;
  Array1D<double> chnsig;
  int stFix,stAdapt,stEnd,nmcmc;

  stFix   = FIXST   ;
  stAdapt = ADPST   ;
  stEnd   = ENDAD   ;
  nmcmc   = MCMCST  ;
  gamma   = GAMMFAC ;

  /* allocate necessary space */
  drivers  = new double[5*nday] ;
  model    = new double[6*nday] ;
  obsdata  = new double[  nday] ;
  obssigma = new double[  nday] ;

  /* Read drivers */
  readDriverData((char *)DRIVERFILE, nday, drivers) ;

  /* Read observations */
  readObs((char *)OBSFILE, nday, obsdata, obssigma);

  /* Read start values - start from nominal values */
  Array1D<double> chnstrt(npar);
  read_datafile_1d(chnstrt,"parnom.dat");
  
  /* Read bounds */
  Array2D<double> chnbnds;
  read_datafileVS(chnbnds,"parbnd.dat");
  assert((int) chnbnds.XSize() == npar );

  /* Initialize diagonal of covariance matrix */
  chnsig.Resize(npar);
  for (int i=0; i < npar; i++) 
    chnsig(i) = (chnbnds(i,1)-chnbnds(i,0))/SIGFAC;

  /* setup parameter structure */  
  params.drivers  = drivers  ;
  params.model    = model    ;
  params.obsdata  = obsdata  ;
  params.obssigma = obssigma ;
  params.dayst    = dayst   ;
  params.dayen    = dayen   ;
  params.nday     = nday    ;
  params.ncycles  = ncycles ;
  params.npar     = npar    ;  

  MCMC mchain(logDalecPost,(void *)&params);
  mchain.setChainDim(npar);

  /* Set bounds */
  for (int i=0; i < npar; i++) {
    mchain.setLower(chnbnds(i,0),i);
    mchain.setUpper(chnbnds(i,1),i);
  }
        
  /* initialize MCMC parameters */
  mchain.initAMGamma(gamma);
  mchain.initAdaptSteps(stFix,stAdapt,stEnd);
  mchain.initChainPropCovDiag(chnsig);
  mchain.setOutputInfo(string("txt"),string("chain.dat"),SAVE2FILE,SAVE2SCREEN);
    
  // print chain setup
  mchain.printChainSetup();

  // run MCMC
  mchain.runChain(nmcmc, chnstrt);

  // write out MAP parameters
  Array1D<double> mappars;
  mchain.getMode(mappars);
  write_datafile_1d(mappars,"MAPpars.dat");
 
  delete [] drivers;
  delete [] model;
  delete [] obsdata;
  delete [] obssigma;

  return ( 0 ) ;

}

/* 
  Log-posterior function, no model error
*/
#include "logPost.cpp"
/* 
  Log-posterior function, model error: band-diagonal covariance matrix
*/
#include "logPostModErrCorrBand.cpp"


double logDalecPrior(Array1D<double> &chain, void *params) {

  double logPr=0.0;

#ifdef INFORM

  int nrm=8,lnnrm=8;

  double jntFctr = 0.974956 ;

  int nrmList[8] = {0, 7, 9, 10, 11, 12, 13, 17};
  double nrmFctrs[8]  = {0.998365, 0.999968, 0.999998, 0.999205, 0.999571, 0.999571, 0.999571, 0.999999}; 
  double nrmMuSig[16] = {0.7, 0.085,
			 4.0, 0.5,
                         80.0,13.0,
			 7.0, 1.9,
                         2.0, 0.3,
			 2.0, 0.3,
                         0.2, 0.045,
			 5.0, 1.0};

  int lnList[8] = {1, 2, 3, 4,  8,  14, 15, 16};
  double lnFctrs[8] = {0.997621, 0.99797,  0.999749, 0.998907, 0.99437,  0.998907, 0.999484, 0.999484};
  double lnMuSig[16]={-7.87152, 1.15707, 
		      -6.46416, 0.359009, 
		      -9.21643, 0.290978,
		      -6.51833, 0.624036, 
                      -1.93689, 0.604725,
		      -4.21575, 0.624036,
		      -7.32244, 0.43231, 
		      -9.62503, 0.43231};

  /* Add joint between 6 (gdd_min) & 7 (gdd_max) */
  if (chain(5)>chain(6)) return (-1.e80);
  logPr -= (pow(chain(5)-100.0,2)/(2.0*24.0*24.0)+0.5*log(DPI*24.0*24.0));
  logPr -= (pow(chain(6)-200.0,2)/(2.0*45.0*45.0)+0.5*log(DPI*45.0*45.0));
  logPr -= log(jntFctr);

  /* Add normal priors */
  for (int i=0; i<nrm; i++ ) {
    int j = nrmList[i];
    double mu = nrmMuSig[2*i  ];
    double s2 = nrmMuSig[2*i+1]*nrmMuSig[2*i+1];
    logPr -= (pow(chain(j)-mu,2)/(2.0*s2)+0.5*log(DPI*s2));
    logPr -= log(nrmFctrs[i]);
  }

  /* Add log-normal priors */
  for (int i=0; i<lnnrm; i++ ) {
    int j = lnList[i];
    double mu = lnMuSig[2*i  ];
    double s2 = lnMuSig[2*i+1]*lnMuSig[2*i+1];
    logPr -= (pow(log(chain(j))-mu,2)/(2.0*s2)+0.5*log(DPI*s2*chain(j)*chain(j)));
    logPr -= log(lnFctrs[i]);
  }
  
#ifndef NOPLSPRIOR
  // Add priors for C pools
  if (NPAR==26) {
    /* Veg 1 */
    int j = 18;
    double mu = 0.0;
    double s2 = 20.0*20.0;
    logPr -= (pow(chain(j)-mu,2)/(2.0*s2)+0.5*log(DPI*s2));
    logPr -= log(0.5);

    /* Veg 2 */
    j = 19;
    mu = 9000.0;
    s2 = pow(0.2*mu,2);
    logPr -= (pow(chain(j)-mu,2)/(2.0*s2)+0.5*log(DPI*s2));

    /* Veg 3 */
    j = 20;
    mu = 1500.0;
    s2 = pow(0.2*mu,2);
    logPr -= (pow(chain(j)-mu,2)/(2.0*s2)+0.5*log(DPI*s2));

    /* Soil 1 */
    j = 21;
    mu = 2.94686;
    s2 = 0.802669*0.802669;
    logPr -= (pow(log(chain(j))-mu,2)/(2.0*s2)+0.5*log(DPI*s2*chain(j)*chain(j)));

    /* Soil 2 */
    j = 22;
    mu = 8800.0;
    s2 = pow(0.2*mu,2);
    logPr -= (pow(chain(j)-mu,2)/(2.0*s2)+0.5*log(DPI*s2));
  }
#endif

#endif

  return (logPr);

}

void readDriverData(char *dfile, int nday, double *dridata) {

  FILE *fsetup = fopen(dfile,"r") ;
  if ( fsetup ) {
    for ( int i = 0; i < 5*nday; i++ )
      fscanf(fsetup, "%le", &dridata[i]);
    fclose(fsetup) ;
  }
  else {
    printf("readDriverData(): Could not open %s -> Abort!\n",DRIVERFILE);
    fflush(stdout);
    exit(0);
  }

  return ;

}

void readObs(char *ofile, int nday, double *obsdata, double *obssigma) {

  std::string line ;
  double dtmp ;

  ifstream  obsfile ;
  obsfile.open(ofile);
  if ( obsfile.fail()) { 
    cout << "readObs() : could not open " << OBSFILE << endl << flush ; 
    exit(0); // no point continuing if the file didn't open...
  } 
  int iobs = 0;
  std::getline(obsfile, line) ; /* skip first line */
  while (std::getline(obsfile, line))
  {
    std::stringstream ss(line);
    ss >> dtmp >> dtmp >> obsdata[iobs] >> dtmp >> obssigma[iobs] ; /* Get columns 3 & 5 */
    iobs++ ;
  }
  obsfile.close() ;

  return ;

}
