#!/bin/bash

if [ "$#" -ne 3 ]; then
        echo "usage $0 runtype sidx nspls"
        echo "      runtype: dst/dtr"
        echo "      sidx   : Si/Sij/SiT"
        echo "      nspls  : no. of random samples to estimate Sobol indices"
        exit
fi

# Preamble 
vars=(NEE GPP TVC TSC LAI HR)

wdir=${PWD}
runtype=$1
if [ "$runtype" == "dst" ]; then
  npar=18
elif [ "$runtype" == "dtr" ]; then
  npar=23
else
  echo "Unknown runtype: $1"
fi
sidx=$2
nspls=$3

# set directories
wdir=${PWD}
srcdir=${wdir}/../../lib/dalec
spldir=${wdir}/../../lib/spls
scpdir=${wdir}/../../scripts/gsa
datdir=${wdir}/../../data
rundir=${wdir}/${runtype}_${sidx}

# generate samples
echo " -> Generate random parameter samples drawn from prior distributions"
cd ${rundir}
/bin/cp ${spldir}/trdSpls .
/bin/cp ${spldir}/gddSpls .
/bin/cp ${scpdir}/genSpls.sh .
/bin/cp ${datdir}/x2.dat .
/bin/cp ${datdir}/f2.dat .
/bin/cp ${datdir}/x1f1.dat .
./genSpls.sh ${nspls} 2015 1 ${npar} > gsa_log1.txt
./genSpls.sh ${nspls} 5102 2 ${npar} > gsa_log2.txt

# generate extended matrix of samples
echo " -> Generate extended matrix of samples"
/bin/cp ${scpdir}/gsa.py .
echo "import numpy as npy"     >  genspls_${sidx}.py
echo "import gsa"              >> genspls_${sidx}.py
echo "gsa.genSpl_${sidx}(${npar},verb=0)" >> genspls_${sidx}.py
python genspls_${sidx}.py
neval=`wc -l gsaSpl${sidx}.dat | awk '{print $1}'`

# build dalec gsa wrapper
echo " -> Building gsa wrapper"
cd ${srcdir}
make -f makefile.cln
make -f makefile.gsa gsa${npar} NEVAL=${neval}
cd ${wdir}

# copy things to run directory
echo " -> Running DALEC gsa wrapper (it might take a while)"
cd ${rundir}
/bin/cp ${srcdir}/dalec_gsa.exe        .
ln -fs  ${datdir}/US-Ha1_drivers.csv   .
ln -fs  ${datdir}/calendar.txt         .
ln -fs  gsaSpl${sidx}.dat              sgsa.dat
./dalec_gsa.exe >& log_${runtype}_d${ndiag}.txt

for v in ${vars[@]}; do
  /bin/mv dalec_spl_${v}.dat spl${sidx}_${v}.dat 
done

# Compute Sobol indices and make figs
echo " -> Compute Sobol indices"
echo "import numpy as npy"     >  getSobol_${sidx}.py
echo "import gsa"              >> getSobol_${sidx}.py
echo "for qoi in ['NEE','GPP','TVC','TSC']:"                 >> getSobol_${sidx}.py
echo "    meval=npy.genfromtxt('spl${sidx}_'+qoi+'.dat')"    >> getSobol_${sidx}.py
echo "    npy.savez('meval'+qoi+'_${sidx}.npz',meval=meval)" >> getSobol_${sidx}.py

if [ "$sidx" == "Si" ]; then
   echo "    Si=npy.array([gsa.genSens_Si(meval[:,0],${nspls})]).T" >> getSobol_${sidx}.py
   echo "    for i in range(1,meval.shape[1]):"                     >> getSobol_${sidx}.py
   echo "        tmpS = npy.array([gsa.genSens_Si(meval[:,i],${nspls})]).T">> getSobol_${sidx}.py
   echo "        Si=npy.concatenate((Si,tmpS),axis=1)"             >> getSobol_${sidx}.py
   echo "    npy.savetxt('Si_'+qoi+'.dat', Si, fmt='%.6e', delimiter=' ', newline='\n')" >> getSobol_${sidx}.py
fi
if [ "$sidx" == "SiT" ]; then
   echo "    SiT=npy.array([gsa.genSens_SiT(meval[:,0],${nspls})]).T" >> getSobol_${sidx}.py
   echo "    for i in range(1,meval.shape[1]):"                     >> getSobol_${sidx}.py
   echo "        tmpS = npy.array([gsa.genSens_SiT(meval[:,i],${nspls})]).T">> getSobol_${sidx}.py
   echo "        SiT=npy.concatenate((SiT,tmpS),axis=1)"             >> getSobol_${sidx}.py
   echo "    npy.savetxt('SiT_'+qoi+'.dat', SiT, fmt='%.6e', delimiter=' ', newline='\n')" >> getSobol_${sidx}.py
fi

if [ "$sidx" == "Sij" ]; then
   echo "    sobolSi = npy.genfromtxt('../${runtype}_Si/Si_'+qoi+'.dat')"        >> getSobol_${sidx}.py
   echo "    Sij=npy.array([gsa.genSens_Sij(sobolSi[:,1],meval[:,1],${nspls})])" >> getSobol_${sidx}.py
   echo "    for i in range(2,meval.shape[1]):"                                  >> getSobol_${sidx}.py
   echo "        tmpS = npy.array([gsa.genSens_Sij(sobolSi[:,i],meval[:,i],${nspls})])" >> getSobol_${sidx}.py
   echo "        Sij=npy.concatenate((Sij,tmpS))"                                >> getSobol_${sidx}.py
   echo "    npy.savez('Sij_'+qoi+'.npz', Sij=Sij)"                              >> getSobol_${sidx}.py
fi

python getSobol_${sidx}.py

if [ "$sidx" == "Si" ]; then
  ${scpdir}/mkp_sm_${npar}p.py NEE 0 vblock
  ${scpdir}/mkp_sm_${npar}p.py GPP 0 vblock
  ${scpdir}/mkp_sm_${npar}p.py TVC 0 vblock
  ${scpdir}/mkp_sm_${npar}p.py TSC 0 vblock
fi

if [ "$sidx" == "SiT" ]; then
  ${scpdir}/mkp_sm_${npar}p.py NEE 1 vblock
  ${scpdir}/mkp_sm_${npar}p.py GPP 1 vblock
  ${scpdir}/mkp_sm_${npar}p.py TVC 1 vblock
  ${scpdir}/mkp_sm_${npar}p.py TSC 1 vblock
fi

if [ "$sidx" == "Sij" ]; then
  cp ${scpdir}/wrapSij.sh .
  cp ${scpdir}/getSij.py  .
  let "npar1=${npar}-1"
  ./wrapSij.sh NEE ${npar1}
  ./wrapSij.sh GPP ${npar1}
  ./wrapSij.sh TVC ${npar1}
  ./wrapSij.sh TSC ${npar1}
fi

cd ${wdir}

