#!/usr/bin/env python

import os, sys
import numpy as npy
import h5py
import matplotlib.pyplot as plt
from scipy import stats, mgrid, c_, reshape, random, rot90
from matplotlib.ticker import MultipleLocator, FormatStrFormatter

figname='cal_rip_d12.pdf'

#
# Pick the right parameters
#
vnames=["astem","br\_mr","br\_lit","br\_som","dr","gdd\_min","gdd\_max",
  "laimax","leaffall","lma","nue","q10\_mr","q10\_hr","rg\_frac","tleaf",
  "troot","tstem","tsmin"]

#vpick=["gdd\_min", "gdd\_max","leaffall","lma", "nue", "q10\_mr",
#       "br\_mr", "rg\_frac","tstem","q10\_hr","br\_lit"]

vpick=["gdd\_min", "gdd\_max","leaffall","lma", "nue", "q10\_mr",
       "br\_mr", "rg\_frac","br\_lit"]

idx=[vnames.index(i) for i in vpick]

# Read 1D pdfs
x1D=[]
p1D=[]
for i in idx:
    file = h5py.File("dens_"+str(i+1)+'.h5', 'r')   
    x1D.append(npy.array(file["x"  ]))
    p1D.append(npy.array(file["p1D"]))
    file.close()

x2D = []
y2D = []
p2D = []
for j in range(1,len(idx)):
    for i in range(j):
        if ( idx[i] < idx[j] ):
            file = h5py.File("dens_"+str(idx[i]+1)+'_'+str(idx[j]+1)+'.h5', 'r')   
            prm = 0;
        else:
            file = h5py.File("dens_"+str(idx[j]+1)+'_'+str(idx[i]+1)+'.h5', 'r')   
            prm = 1;
        if (prm==1):
            x2D.append(npy.transpose(npy.array(file["y"  ])))
            y2D.append(npy.transpose(npy.array(file["x"  ])))
            p2D.append(npy.transpose(npy.array(file["p2D"])))
        else:
            x2D.append(npy.array(file["x"  ]))
            y2D.append(npy.array(file["y"  ]))
            p2D.append(npy.array(file["p2D"]))
        file.close()

x2D = npy.array(x2D)
y2D = npy.array(y2D)
p2D = npy.array(p2D)

nvars=len(x1D)

# Section 5
# ds is the distance between subplots
# xs,ys are the coordinates (normalized) of the subplot in the lower left corner
# xe,ye are the distances left in the uppper right corner
# ds are distances between subplots
# fsizex, fsizey are figure sizes
# ncont are no of contours for 2D pdfs
xs=0.05; ys=0.05; ds=0.005
xe=0.05; ye=0.05
fsizex=12; fsizey=12;
ncont=20; 
sx=(1-(nvars-1)*ds-xs-xe)/nvars;
sy=(1-(nvars-1)*ds-ys-ye)/nvars;

fs1=16

majorFormatter = FormatStrFormatter('%6.0e')

fig = plt.figure(figsize=(fsizex,fsizey))

# Section 5.1
subs=[]
# add diagonal plots
for i in range(nvars):
    subs.append(fig.add_axes([xs+i*(sx+ds),ys+(nvars-1-i)*(sy+ds),sx,sy]))

# add lower triangular plots
for i in range(nvars-1):
    for j in range(i+1):
        #print j,(nvars-2-i)
        subs.append(fig.add_axes([xs+j*(sx+ds),ys+(nvars-2-i)*(sy+ds),sx,sy]))

subsnp=npy.array(subs)

# Plot 1D pdfs
for i in range(nvars):
    pl1d,=subsnp[i].plot(x1D[i],p1D[i])

# Plot 2D pdfs
for i in range(nvars*(nvars-1)/2):
    pl2d=subsnp[nvars+i].contour(x2D[i],y2D[i],p2D[i],ncont)

# Section 5.2
# no ticks and ticklabels
for i in range(nvars,len(subsnp)):
    xti=subsnp[i].set_xticks([]); 
    yti=subsnp[i].set_yticks([]); 

# for diagonal plots only put xmin and xmax
#for i in range(1,nvars+1):
#    subsnp[i-1].set_xticks([d0[nskip:,i].min(),d0[nskip:,i].max()]); 

# Set y labels on the right for diagonal plots
for i in range(nvars):
    subsnp[i].xaxis.tick_top()
    subsnp[i].yaxis.tick_right()
    subsnp[i].yaxis.set_label_position("right")
    #subsnp[i].set_ylabel(r'$'+str(i+1)+'$', fontsize=fs1, rotation=0)

icnt=0
for i in range(len(subsnp)-nvars+1,len(subsnp)):
    icnt+=1
    subsnp[i].set_xlabel(r'$'+vpick[icnt-1]+'$', fontsize=fs1)

icnt=nvars
for i in range(nvars,2*nvars-1):
    icnt+=i-nvars
    subsnp[icnt].set_ylabel(r'$'+vpick[i-nvars+1]+'$', fontsize=fs1, rotation=90)

subsnp[0].yaxis.set_label_position("left")
subsnp[0].set_ylabel(r'$'+vpick[0]+'$', fontsize=fs1, rotation=90)
subsnp[nvars-1].set_xlabel(r'$'+vpick[nvars-1]+'$', fontsize=fs1)
 
# Custom labels
subsnp[0].set_xticks([10,25,40]);
subsnp[0].set_yticks([0.1,0.2]);
subsnp[1].set_xticks([200,240]);
subsnp[1].set_yticks([0.03, 0.06]);
subsnp[2].set_xticks([0.035,0.045]);
subsnp[2].set_yticks([100,200]);
subsnp[3].set_xticks([100,140]);
subsnp[3].set_yticks([0.02,0.04]);
subsnp[4].set_xticks([6,10,14]);
subsnp[4].set_yticks([0.2,0.4]);
subsnp[5].set_xticks([1,1.1,1.2]);
subsnp[5].set_yticks([8,16]);
subsnp[6].set_xticks([0.0025,0.004]);
subsnp[6].set_yticks([600,1200]);
subsnp[7].set_xticks([0.1,0.2]);
subsnp[7].set_yticks([10,20]);
subsnp[8].set_xticks([0.002,0.004]);
subsnp[8].set_yticks([500,1000]);

#plt.show()
plt.savefig(figname)

#
#  1D pdf's only
#
vall=["gdd\_min","gdd\_max","tsmin","laimax","leaffall","lma","nue",
      "q10\_mr","br\_mr","rg\_frac","astem","tstem","troot","tleaf",
      "q10\_hr","br\_lit","br\_som","dr"]

# Keep only the variables not picked above
idx=[vall.index(i) for i in vpick]
vidp = npy.delete(vall, idx)

# For the remainining variables get their index in the saved pdf's
idx=[vnames.index(i) for i in vidp]
x1D=[]
p1D=[]
for i in idx:
    file = h5py.File("dens_"+str(i+1)+'.h5', 'r')   
    x1D.append(npy.array(file["x"  ]))
    p1D.append(npy.array(file["p1D"]))
    file.close()

nvars=len(x1D)

# Plot 1D pdfs
fs1=24
lw1=3
for i in range(nvars):
    #print "pdf1D for ",vnames[idx[i]]
    fig = plt.figure(figsize=(6,4))
    ax1=fig.add_axes([0.2,0.16,0.75,0.79])
    pl1d,=ax1.plot(x1D[i],p1D[i],linewidth=lw1)
    xlb=ax1.set_xlabel(r'$'+vidp[i]+'$', fontsize=fs1)
    ylb=ax1.set_ylabel(r'$p('+vidp[i]+')$', fontsize=fs1, rotation=90)
    if (i==0):
       xtk=ax1.set_xticks([7.6,7.8,8,8.2,8.4])
       ytk=ax1.set_yticks([0,2,4,6])
    elif (i==1):
       xtk=ax1.set_xticks([2.0,2.05,2.1,2.15])
       ytk=ax1.set_yticks([0,20,40,60])        
    elif (i==2):
       xtk=ax1.set_xticks([0.4,0.8,1.2])
       ytk=ax1.set_yticks([0,2,4])        
    elif (i==3):
       xlm=ax1.set_xlim([0,0.00033])	 	 
       xtk=ax1.set_xticks([0.0001,0.0002,0.0003])
       ytk=ax1.set_yticks([0,5000,10000])        
       ax1.set_yticklabels(["0","$5\cdot 10^3$","$10\cdot 10^3$"],fontsize=fs1-4,rotation=60)
    elif (i==4):
       xlm=ax1.set_xlim([0,0.0033])	 	 
       xtk=ax1.set_xticks([0,0.001,0.002,0.003])
       ytk=ax1.set_yticks([0,500,1000,1500])
    #   ax1.set_xticklabels(["0","$10^{-4}$","$2\cdot 10^{-4}$"],fontsize=fs1-4)
    #   ax1.set_yticklabels(["0","$10^4$","$2\cdot 10^4$"],fontsize=fs1-4)
    elif (i==5):
       xlm=ax1.set_xlim([0,0.12])	 	 
       xtk=ax1.set_xticks([0,0.05,0.1])
       ytk=ax1.set_yticks([0,30,60])
    #   ax1.set_xticklabels(["0","$10^{-4}$","$2\cdot 10^{-4}$"],fontsize=fs1-4)
    #   ax1.set_yticklabels(["0","$10^4$","$2\cdot 10^4$"],fontsize=fs1-4)
    elif (i==6):
       xlm=ax1.set_xlim([1.6,3.4])	 	 
       xtk=ax1.set_xticks([2,2.5,3])
       ytk=ax1.set_yticks([0,1,2])
    elif (i==7):
       xlm=ax1.set_xlim([0,0.00033])	 	 
       xtk=ax1.set_xticks([0,0.0001,0.0002,0.0003])
       ytk=ax1.set_yticks([0,6000,12000])
       ax1.set_yticklabels(["0","$6\cdot 10^3$","$12\cdot 10^3$"],fontsize=fs1-4,rotation=60)
    elif (i==8):
       xlm=ax1.set_xlim([0,0.0062])	 	 
       xtk=ax1.set_xticks([0,0.002,0.004,0.006])
       ytk=ax1.set_yticks([0,600,1200])
       #ax1.set_yticklabels(["0","$6\cdot 10^3$","$12\cdot 10^3$"],fontsize=fs1-4,rotation=60)
    for tick in ax1.xaxis.get_major_ticks():
        tick.label.set_fontsize(fs1-4)
    for tick in ax1.yaxis.get_major_ticks():
        tick.label.set_fontsize(fs1-4)
    #plt.show()
    plt.savefig("pdf1D_"+vidp[i].replace("\\","")+"_18p.pdf")
    plt.close(fig)
